@@ -1,3 +1,7 @@
+from . import dsp
+from . import equalizer
+from . import common
+
 import numpy as np
 import logging
 import itertools
@@ -5,10 +9,6 @@ import collections
 
 log = logging.getLogger(__name__)
 
-from . import dsp
-from . import equalizer
-from . import common
-
 
 class Detector(object):
 
@@ -17,8 +17,7 @@ class Detector(object):
     CARRIER_DURATION = sum(equalizer.prefix)
     CARRIER_THRESHOLD = int(0.9 * CARRIER_DURATION)
     SEARCH_WINDOW = int(0.1 * CARRIER_DURATION)
-
-    TIMEOUT = 10.0  # [seconds]
+    START_PATTERN_LENGTH = SEARCH_WINDOW // 4
 
     def __init__(self, config, pylab):
         self.freq = config.Fc
@@ -26,7 +25,7 @@ class Detector(object):
         self.Nsym = config.Nsym
         self.Tsym = config.Tsym
         self.maxlen = config.baud  # 1 second of symbols
-        self.max_offset = self.TIMEOUT * config.Fs
+        self.max_offset = config.timeout * config.Fs
         self.plt = pylab
 
     def _wait(self, samples):
@@ -66,7 +65,7 @@ class Detector(object):
         bufs.append(np.array(trailing))
 
         buf = np.concatenate(bufs)
-        offset = self.find_start(buf, duration=self.CARRIER_DURATION)
+        offset = self.find_start(buf)
         start_time += (offset / self.Nsym - self.SEARCH_WINDOW) * self.Tsym
         log.debug('Carrier starts at %.3f ms', start_time * 1e3)
 
@@ -76,14 +75,23 @@ class Detector(object):
         amplitude, freq_err = self.estimate(buf[:prefix_length])
         return itertools.chain(buf, samples), amplitude, freq_err
 
-    def find_start(self, buf, duration):
-        filt = dsp.FIR(dsp.exp_iwt(self.omega, self.Nsym))
-        p = np.abs(list(filt(buf))) ** 2
-        p = np.cumsum(p)[self.Nsym-1:]
-        p = np.concatenate([[0], p])
-        length = (duration - 1) * self.Nsym
-        correlations = np.abs(p[length:] - p[:-length])
-        offset = np.argmax(correlations)
+    def find_start(self, buf):
+        carrier = dsp.exp_iwt(self.omega, self.Nsym)
+        carrier = np.tile(carrier, self.START_PATTERN_LENGTH)
+        zeroes = carrier * 0.0
+        signal = np.concatenate([zeroes, carrier])
+        signal = (2 ** 0.5) * signal / dsp.norm(signal)
+
+        coeffs = []
+        for i in range(len(buf) - len(signal)):
+            b = buf[i:i+len(signal)]
+            norm_b = dsp.norm(b)
+            c = (np.abs(np.dot(b, signal)) / norm_b) if norm_b else 0.0
+            coeffs.append(c)
+
+        index = np.argmax(coeffs)
+        log.info('Carrier coherence: %.3f%%', coeffs[index] * 100)
+        offset = index + len(zeroes)
         return offset
 
     def estimate(self, buf, skip=5):
@@ -93,7 +101,7 @@ class Detector(object):
         symbols = np.array(symbols[skip:-skip])
 
         amplitude = np.mean(np.abs(symbols))
-        log.debug('Carrier symbols amplitude : %.3f', amplitude)
+        log.info('Carrier symbols amplitude : %.3f', amplitude)
 
         phase = np.unwrap(np.angle(symbols)) / (2 * np.pi)
         indices = np.arange(len(phase))
@@ -103,9 +111,6 @@ class Detector(object):
         self.plt.plot(indices, a * indices + b)
 
         freq_err = a / (self.Tsym * self.freq)
-        last_phase = a * indices[-1] + b
-        log.debug('Current phase on carrier: %.3f', last_phase)
-        log.debug('Frequency error: %.3f ppm', freq_err * 1e6)
+        log.info('Frequency error: %.3f ppm', freq_err * 1e6)
         self.plt.title('Frequency drift: {0:.3f} ppm'.format(freq_err * 1e6))
-
         return amplitude, freq_err
