@@ -1,3 +1,8 @@
+from . import dsp
+from . import common
+from . import framing
+from . import equalizer
+
 import numpy as np
 import logging
 import itertools
@@ -6,14 +11,6 @@ import time
 
 log = logging.getLogger(__name__)
 
-from . import stream
-from . import dsp
-from . import sampling
-from . import common
-from . import framing
-from . import equalizer
-from . import detect
-
 
 class Receiver(object):
 
@@ -26,6 +23,7 @@ class Receiver(object):
         self.Nsym = config.Nsym
         self.Tsym = config.Tsym
         self.iters_per_update = 100  # [ms]
+        self.iters_per_report = 1000  # [ms]
         self.modem_bitrate = config.modem_bps
         self.equalizer = equalizer.Equalizer(config)
         self.carrier_index = config.carrier_index
@@ -43,8 +41,10 @@ class Receiver(object):
         self.plt.subplot(1, 2, 2)
         self.plt.plot(np.abs(S))
         self.plt.plot(equalizer.prefix)
-        if any(bits != equalizer.prefix):
-            raise ValueError('Incorrect prefix')
+        errors = (bits != equalizer.prefix)
+        if any(errors):
+            msg = 'Incorrect prefix: {0} errors'.format(sum(errors))
+            raise ValueError(msg)
         log.debug('Prefix OK')
 
     def _train(self, sampler, order, lookahead):
@@ -60,7 +60,7 @@ class Receiver(object):
 
         coeffs = equalizer.train(
             signal=signal[prefix:-postfix],
-            expected=train_signal,
+            expected=np.concatenate([train_signal, np.zeros(lookahead)]),
             order=order, lookahead=lookahead
         )
 
@@ -68,6 +68,7 @@ class Receiver(object):
         self.plt.plot(np.arange(order+lookahead), coeffs)
 
         equalization_filter = dsp.FIR(h=coeffs)
+        # Pre-load equalization filter with the signal (+lookahead)
         equalized = list(equalization_filter(signal))
         equalized = equalized[prefix+lookahead:-postfix+lookahead]
         self._verify_training(equalized, train_symbols)
@@ -81,10 +82,9 @@ class Receiver(object):
         error_rate = errors.sum() / errors.size
 
         errors = np.array(symbols - train_symbols)
-        rms = lambda x: (np.mean(np.abs(x) ** 2, axis=0) ** 0.5)
 
-        noise_rms = rms(errors)
-        signal_rms = rms(train_symbols)
+        noise_rms = dsp.rms(errors)
+        signal_rms = dsp.rms(train_symbols)
         SNRs = 20.0 * np.log10(signal_rms / noise_rms)
 
         self.plt.figure()
@@ -94,14 +94,9 @@ class Receiver(object):
                                 '$F_c = {0} Hz$'.format(freq), index=i)
         assert error_rate == 0, error_rate
 
-    def _demodulate(self, sampler, symbols):
+    def _bitstream(self, symbols, error_handler):
         streams = []
         symbol_list = []
-        errors = {}
-
-        def error_handler(received, decoded, freq):
-            errors.setdefault(freq, []).append(received / decoded)
-
         generators = common.split(symbols, n=len(self.omegas))
         for freq, S in zip(self.frequencies, generators):
             equalized = []
@@ -110,33 +105,51 @@ class Receiver(object):
 
             freq_handler = functools.partial(error_handler, freq=freq)
             bits = self.modem.decode(S, freq_handler)  # list of bit tuples
-            streams.append(bits)  # stream per frequency
+            streams.append(bits)  # bit stream per frequency
 
+        return common.izip(streams), symbol_list
+
+    def _demodulate(self, sampler, symbols):
+        symbol_list = []
+        errors = {}
+        noise = {}
+
+        def _handler(received, decoded, freq):
+            errors.setdefault(freq, []).append(received / decoded)
+            noise.setdefault(freq, []).append(received - decoded)
+
+        stream, symbol_list = self._bitstream(symbols, _handler)
         self.stats['symbol_list'] = symbol_list
         self.stats['rx_bits'] = 0
         self.stats['rx_start'] = time.time()
 
         log.info('Starting demodulation')
-        for i, block in enumerate(common.izip(streams), 1):
-            for bits in block:
+        for i, block_of_bits in enumerate(stream, 1):
+            for bits in block_of_bits:
                 self.stats['rx_bits'] = self.stats['rx_bits'] + len(bits)
                 yield bits
 
             if i % self.iters_per_update == 0:
-                self._update_sampler(i, errors, sampler)
+                self._update_sampler(errors, sampler)
 
-    def _update_sampler(self, iter_index, errors, sampler):
+            if i % self.iters_per_report == 0:
+                self._report_progress(noise, sampler)
+
+    def _update_sampler(self, errors, sampler):
         err = np.array([e for v in errors.values() for e in v])
         err = np.mean(np.angle(err))/(2*np.pi) if len(err) else 0
         errors.clear()
 
-        duration = time.time() - self.stats['rx_start']
         sampler.freq -= 0.01 * err * self.Tsym
         sampler.offset -= err
+
+    def _report_progress(self, noise, sampler):
+        e = np.array([e for v in noise.values() for e in v])
+        noise.clear()
         log.debug(
-            'Got  %10.3f kB, realtime: %6.2f%%, drift: %+5.2f ppm',
+            'Got  %10.3f kB, SNR: %5.2f dB, drift: %+5.2f ppm',
             self.stats['rx_bits'] / 8e3,
-            duration * 100.0 / (iter_index * self.Tsym),
+            -10 * np.log10(np.mean(np.abs(e) ** 2)),
             (1.0 - sampler.freq) * 1e6
         )
 
@@ -144,17 +157,15 @@ class Receiver(object):
         symbols = dsp.Demux(sampler, omegas=self.omegas, Nsym=self.Nsym)
         self._prefix(symbols, gain=gain)
 
-        filt = self._train(sampler, order=20, lookahead=20)
+        filt = self._train(sampler, order=10, lookahead=10)
         sampler.equalizer = lambda x: list(filt(x))
 
         bitstream = self._demodulate(sampler, symbols)
         bitstream = itertools.chain.from_iterable(bitstream)
 
-        data = framing.decode(bitstream)
-        for chunk in common.iterate(data=data, size=256,
-                                    truncate=False, func=bytearray):
-            output.write(chunk)
-            self.output_size += len(chunk)
+        for frame in framing.decode_frames(bitstream):
+            output.write(frame)
+            self.output_size += len(frame)
 
     def report(self):
         if self.stats:
@@ -192,47 +203,3 @@ class Receiver(object):
         self.plt.axis('equal')
         self.plt.axis(np.array([-1, 1, -1, 1])*1.1)
         self.plt.title(title)
-
-
-class Dumper(object):
-    def __init__(self, src, dst):
-        self.src = src
-        self.dst = dst
-
-    def read(self, size):
-        data = self.src.read(size)
-        self.dst.write(data)
-        return data
-
-
-def main(config, src, dst, dump_audio=None, pylab=None):
-    if dump_audio:
-        src = Dumper(src, dump_audio)
-    reader = stream.Reader(src, data_type=common.loads)
-    signal = itertools.chain.from_iterable(reader)
-
-    log.debug('Skipping %.3f seconds', config.skip_start)
-    common.take(signal, int(config.skip_start * config.Fs))
-
-    pylab = pylab or common.Dummy()
-    detector = detect.Detector(config=config, pylab=pylab)
-    receiver = Receiver(config=config, pylab=pylab)
-    try:
-        log.info('Waiting for carrier tone: %.1f kHz', config.Fc / 1e3)
-        signal, amplitude, freq_error = detector.run(signal)
-
-        freq = 1 / (1.0 + freq_error)  # receiver's compensated frequency
-        log.debug('Frequency correction: %.3f ppm', (freq - 1) * 1e6)
-
-        gain = 1.0 / amplitude
-        log.debug('Gain correction: %.3f', gain)
-
-        sampler = sampling.Sampler(signal, sampling.Interpolator(), freq=freq)
-        receiver.run(sampler, gain=1.0/amplitude, output=dst)
-        return True
-    except Exception:
-        log.exception('Decoding failed')
-        return False
-    finally:
-        dst.flush()
-        receiver.report()
