@@ -1,18 +1,17 @@
+from . import common
+from . import equalizer
+from . import dsp
+
 import numpy as np
 import logging
 import itertools
 
 log = logging.getLogger(__name__)
 
-from . import common
-from . import stream
-from . import framing
-from . import equalizer
-from . import dsp
-
 
 class Sender(object):
-    def __init__(self, fd, config):
+    def __init__(self, fd, config, gain=1.0):
+        self.gain = gain
         self.offset = 0
         self.fd = fd
         self.modem = dsp.MODEM(config.symbols)
@@ -24,7 +23,7 @@ class Sender(object):
         self.equalizer = equalizer.Equalizer(config)
 
     def write(self, sym):
-        sym = np.array(sym)
+        sym = np.array(sym) * self.gain
         data = common.dumps(sym)
         self.fd.write(data)
         self.offset += len(sym)
@@ -48,30 +47,3 @@ class Sender(object):
             if i % self.iters_per_report == 0:
                 total_bits = i * Nfreq * self.modem.bits_per_symbol
                 log.debug('Sent %10.3f kB', total_bits / 8e3)
-
-
-def main(config, src, dst):
-    sender = Sender(dst, config=config)
-    Fs = config.Fs
-
-    # pre-padding audio with silence
-    sender.write(np.zeros(int(Fs * config.silence_start)))
-
-    sender.start()
-
-    training_duration = sender.offset
-    log.info('Sending %.3f seconds of training audio', training_duration / Fs)
-
-    reader = stream.Reader(src, eof=True)
-    data = itertools.chain.from_iterable(reader)
-    bits = framing.encode(data)
-    log.info('Starting modulation')
-    sender.modulate(bits=bits)
-
-    data_duration = sender.offset - training_duration
-    log.info('Sent %.3f kB @ %.3f seconds',
-             reader.total / 1e3, data_duration / Fs)
-
-    # post-padding audio with silence
-    sender.write(np.zeros(int(Fs * config.silence_stop)))
-    return True
